export type ID = number | string | bigint;
export type Component<E> = keyof E;
export type ComponentUpdater<T> = T | ((value: T) => T);
export type ComponentUpdaters<E, C extends keyof E> = [
    C,
    ComponentUpdater<E[C]>
];
export type ComponentUpdate<E, C extends keyof E> = [C, E[C]];
export interface BaseEntity {
    id: ID | undefined;
}
export type EntitiesCallback<Entity> = (entities: Set<Entity>) => void | Promise<void>;
export type EntityCallback<Entity> = (entity: Entity) => void | Promise<void>;
export type EntityUpdateCallback<Entity> = (entity: Entity, updates?: ComponentUpdate<Entity, Component<Entity>>[]) => void | Promise<void>;
interface Query<Entity extends {} = any> {
    components: (keyof Entity)[];
    exclude?: (keyof Entity)[];
    entities: Set<Entity>;
    subscriptions: Set<EntitiesCallback<Entity>>;
    updates: Set<EntityUpdateCallback<Entity>>;
}
export interface Config<Entity> {
    getId: (e: Entity) => ID;
    parallel: boolean;
    asyncUpdates: boolean;
    generateId: () => ID;
    onBefore: (...args: any[]) => void | Promise<void>;
    onAfter: (...args: any[]) => void | Promise<void>;
}
export declare function getComponent<Entity extends {}, C extends keyof Entity>(entity: Entity, component: C): Entity[C];
export declare function hasComponent<Entity extends {} = any>(entity: Entity, component: Component<Entity>): boolean;
export declare function hasSomeComponents<Entity extends {} = any>(entity: Entity, components: (keyof Entity)[]): boolean;
export declare function hasComponents<Entity extends {} = any>(entity: Entity, components: (keyof Entity)[]): boolean;
export declare function generateId(): number;
export declare function makeQueryKey<Entity extends {} = any>(components: (keyof Entity)[]): string;
export declare class World<Entity extends {} = any> {
    protected config: Config<Entity>;
    protected _entities: Map<ID, Entity>;
    protected systems: [Function, Set<Entity>][];
    protected queries: Map<string, Query<Entity>>;
    protected _onCreate: Set<EntityCallback<Entity>>;
    protected _onUpdate: Set<EntityUpdateCallback<Entity>>;
    protected _onRemove: Set<EntityCallback<Entity>>;
    get size(): number;
    get entities(): IterableIterator<Entity>;
    constructor(config?: Partial<Config<Entity>>);
    protected queryWithKey(key: string, components: Component<Entity>[], opts?: {
        exclude?: Component<Entity>[];
        persist?: Boolean;
    }): Set<Entity>;
    get(id: ID): Entity | undefined;
    add: (entity: Entity) => Entity;
    create(entity: Entity): Entity;
    remove(entity: Entity): Entity;
    clear(): void;
    query(components: Component<Entity>[], opts?: {
        exclude?: Component<Entity>[];
        persist?: Boolean;
    }): Set<Entity>;
    register(system: Function, components: Component<Entity>[], exclude?: Component<Entity>[]): void;
    addComponents(entity: Entity, components: Partial<Entity>): void;
    removeComponents(entity: Entity, components?: Component<Entity>[]): void;
    run<Args extends readonly any[]>(...args: Args): Promise<void>;
    subscribe(components: Component<Entity>[], callback: EntitiesCallback<Entity>, opts?: {
        emit?: boolean;
        exclude?: Component<Entity>[];
    }): () => void;
    makeSubscription(components: Component<Entity>[], opts?: {
        emit?: boolean;
        exclude?: Component<Entity>[];
    }): (cb: EntitiesCallback<Entity>) => () => void;
    onEntityCreated(cb: EntityCallback<Entity>): () => void;
    onEntityUpdated(cb: EntityCallback<Entity>): () => void;
    onEntityRemoved(cb: EntityCallback<Entity>): () => void;
    onUpdate(components: Component<Entity>[], callback: EntityUpdateCallback<Entity>): () => void;
    makeUpdateHandler(components: Component<Entity>[]): (cb: EntityUpdateCallback<Entity>) => () => void;
    unsubscribe(components: Component<Entity>[], callback: EntitiesCallback<Entity>): void;
    unsubscribeUpdate(components: Component<Entity>[], callback: EntityUpdateCallback<Entity>): void;
    update(entity: Entity, components: Partial<Entity>): Entity;
    updateComponent<T extends Component<Entity>>(entity: Entity, component: T, update: ComponentUpdater<Entity[T]>): Entity[T];
    updateComponents<C extends keyof Entity>(entity: Entity, updates: ComponentUpdaters<Entity, C>[]): ComponentUpdate<Entity, C>[];
    private _update;
    private _handleUpdates;
    private _handleUpdateCallbacks;
    private _handleRemoveCallbacks;
}
export {};
